package szakdolgozat;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;

public class WorkspacePanel extends JPanel{

	/**
	 * 
	 */
	private static final long serialVersionUID = -3532071370434428338L;
	List<Object> components;
	List<Object> transformedShapes;	//linear
	List<Object> nonLinearTransformedShapes;	//nonlinear

	Map<Object,Color> coloringTransformedShapes;
	Map<Object,Color> coloringNonLinTransformedShapes;


	boolean isSaved;
	boolean isChanged=false;
	boolean isTransformed=false;
	Color linTransformationColor;
	Color nonlinTransformationColor;
	int arrowwidth = 6;
	int arrowlength = 6;
	static int rateSpace=10;
	int rateLine = 3;
	int space = 10;
	Point A;
	Point B;
	Point C;
	Point D;
	Point O;
	boolean isgrid=false;
	public JPopupMenu popup;

	public void setRateLine(int rl){
		rateLine=rl;
	}

	public void setRateSpace(int rt){
		rateSpace=rt;
	}

	public WorkspacePanel() {
		super();
		setOpaque(true);
		setPreferredSize(new Dimension(550,560));
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));

		components=null;
		transformedShapes=null;
		nonLinearTransformedShapes=null;

		coloringTransformedShapes=null;
		coloringNonLinTransformedShapes=null;

		isSaved=false;

		popup = new JPopupMenu();

		JMenuItem gridOnItem = new JMenuItem("Rácsozás be");
		popup.add(gridOnItem);
		gridOnItem.setHorizontalTextPosition(JMenuItem.RIGHT);
		gridOnItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				isgrid=true;
				repaint();

			}
		});

		JMenuItem gridOffItem = new JMenuItem("Rácsozás ki");
		popup.add(gridOffItem);
		gridOffItem.setHorizontalTextPosition(JMenuItem.RIGHT);
		gridOffItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				isgrid=false;
				repaint();
			}
		});

		final JMenuItem zoomInItem = new JMenuItem("Nagyítás");
		popup.add(zoomInItem);
		zoomInItem.setHorizontalTextPosition(JMenuItem.RIGHT);
		if(rateSpace==1000){
			zoomInItem.setEnabled(false);
		}


		final JMenuItem zoomOutItem = new JMenuItem("Kicsinyítés");
		popup.add(zoomOutItem);

		zoomOutItem.setHorizontalTextPosition(JMenuItem.RIGHT);
		if(rateSpace==10){
			zoomOutItem.setEnabled(false);
		}

		zoomOutItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if(rateSpace>10){
					rateSpace/=10;
					zoomInItem.setEnabled(true);
				}
				repaint();

			}
		});

		zoomInItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				if(rateSpace<1000){
					rateSpace*=10;
					zoomOutItem.setEnabled(true);

				}
				repaint();

			}
		});

		popup.setLabel("");
		popup.setBorder(new BevelBorder(BevelBorder.RAISED));

		linTransformationColor=Color.RED;
		nonlinTransformationColor=Color.BLUE;

		addMouseListener(new MousePopupListener(popup, this));

	}

	public void setLinTransformationColor(Color color){
		linTransformationColor=color;
	}

	public void setNonLinTransformationColor(Color color){
		nonlinTransformationColor=color;
	}

	public Point calculatePoint(Point2D p, int rateSpace, Point O){
		double x=rateSpace*p.getX();
		double y=rateSpace*p.getY();
		return new Point(O.x + (int)(Math.round(x)),O.y - (int)(Math.round(y)));
	}

	public int calculateCoordinateX(double x, int rateSpace, Point O){
		return O.x+(int)(rateSpace*x);
	}

	public int calculateCoordinateY(double y, int rateSpace, Point O){
		return O.y-(int)(rateSpace*y);
	}

	public static boolean isSzam(char c){
		if((c>='0' && c<='9') || c=='.' || c=='-'){
			return true;
		}
		else{
			return false;
		}
	}

	public void addNewComponent(Object newobj){
		isChanged=true;
		if(components == null){
			components = new ArrayList<Object>();
		}
		components.add(newobj);
		//repaint();
	}

	public void addComponentsToComponents(ArrayList<Object> newlist){
		if(components==null){
			components=new ArrayList<Object>();
		}
		for(int i=0;i<newlist.size();i++){
			components.add(newlist.get(i));
		}
		this.repaint();
	}

	public List<Object> getArrayList(){
		return components;
	}

	public void drawPoint(Graphics2D g2, Point2D.Double P, int pointLine, int rateSpace, Point O){
		Point newPoint = calculatePoint(P, rateSpace, O);
		g2.drawLine(newPoint.x+pointLine, newPoint.y-pointLine, newPoint.x-pointLine, newPoint.y+pointLine);
		g2.drawLine(newPoint.x-pointLine, newPoint.y-pointLine, newPoint.x+pointLine, newPoint.y+pointLine);

	}

	public void addComponentsFromFile(File filename){
		components=null;

		components=new ArrayList<Object>();
		try {
			System.setProperty("file.encoding", "UTF8"); //karakterkódolás beállítása
			BufferedReader br = new BufferedReader(new FileReader(filename));
			String sor;
			List<String> points=new ArrayList<String>();
			List<String> lines=new ArrayList<String>();
			List<String> polygons=new ArrayList<String>();
			List<String> circles=new ArrayList<String>();

			sor=br.readLine();
			boolean isStop=false;
			if(sor!=null && sor.equals("Point")){
				while(!isStop){
					String point="";
					while(!(sor=br.readLine()).equals("*")){
						if(sor.equals("***")){
							isStop=true;
							break;
						}
						else{
							point+=sor;
							point+="*";
						}
					}
					points.add(point);
				}
			}

			sor=br.readLine();
			isStop=false;
			if(sor!=null && sor.equals("Line")){
				while(!isStop){
					String Vonal="";
					while(!(sor=br.readLine()).equals("*")){
						if(sor.equals("***")){
							isStop=true;
							break;
						}
						else{
							Vonal+=sor;
							Vonal+="*";
						}
					}
					lines.add(Vonal);
				}
			}

			sor=br.readLine();
			isStop=false;
			if(sor!=null && sor.equals("Polygon")){
				while(!isStop){
					String Poligon="";
					while(!(sor=br.readLine()).equals("*")){
						if(sor.equals("***")){
							isStop=true;
							break;
						}
						else{
							Poligon+=sor;
							Poligon+="*";
						}
					}
					polygons.add(Poligon);
				}
			}

			sor=br.readLine();
			isStop=false;
			if(sor!=null && sor.equals("Circle")){
				while(!isStop){
					String Kor="";
					while(!(sor=br.readLine()).equals("*")){
						if(sor.equals("***")){
							isStop=true;
							break;
						}
						else{
							Kor+=sor;
							Kor+="*";
						}
					}
					circles.add(Kor);
				}
			}

			br.close();

			for(int i=0;i<points.size();i++){
				if(!points.get(i).equals("")){
					String[] strings = points.get(i).split("\\*");
					String nPoint_x = "";
					String nPoint_y = "";
					Point2D.Double newPoint=null;
					for (String string : strings) {
						System.out.println(string);

						int k;
						for(k=1;k<string.length() && isSzam(string.charAt(k));){
							nPoint_x+=string.charAt(k);
							k++;
						}
						k++;
						System.out.println(nPoint_x);
						for(;k<string.length() && isSzam(string.charAt(k));){
							nPoint_y+=string.charAt(k);
							k++;
						}
						System.out.println(nPoint_y);
						double x=Double.parseDouble(nPoint_x);
						double y=Double.parseDouble(nPoint_y);

						newPoint = new Point2D.Double(x,y);
					}
					components.add(newPoint);
				}
			}

			for(int i=0;i<lines.size();i++){
				if(!lines.get(i).equals("")){
					String[] strings = lines.get(i).split("\\*");

					List<Point2D.Double> pontok=new ArrayList<Point2D.Double>();
					pontok.clear();
					for (String string : strings) {
						String nPoint_x = "";
						String nPoint_y = "";
						System.out.println(string);
						int k;
						for(k=1;k<string.length() && isSzam(string.charAt(k));){
							nPoint_x+=string.charAt(k);
							k++;
						}
						k++;
						System.out.println(nPoint_x);
						for(;k<string.length() && isSzam(string.charAt(k));){
							nPoint_y+=string.charAt(k);
							k++;
						}
						System.out.println(nPoint_y);

						double x=Double.parseDouble(nPoint_x);
						double y=Double.parseDouble(nPoint_y);

						Point2D.Double newPoint=new Point2D.Double(x, y);
						pontok.add(newPoint);
					}
					Line2D.Double v=new Line2D.Double(pontok.get(0),pontok.get(1));
					components.add(v);
				}
			}

			for(int i=0;i<polygons.size();i++){
				if(!polygons.get(i).equals("")){
					String[] strings = polygons.get(i).split("\\*");
					ArrayList<Double> xcoordinates = new ArrayList<Double>();
					ArrayList<Double> ycoordinates = new ArrayList<Double>();
					System.out.println("FIGYEEELJ");
					for (String string : strings) {
						String nPoint_x = "";
						String nPoint_y = "";
						System.out.println(string);
						int k;
						for(k=1;k<string.length() && isSzam(string.charAt(k));){
							nPoint_x+=string.charAt(k);
							k++;
						}
						k++;
						System.out.println(nPoint_x);
						for(;k<string.length() && isSzam(string.charAt(k));){
							nPoint_y+=string.charAt(k);
							k++;
						}
						System.out.println(nPoint_y);

						double x=Double.parseDouble(nPoint_x);
						double y=Double.parseDouble(nPoint_y);

						xcoordinates.add(x);
						ycoordinates.add(y);
					}
					//double[] pontok_x= new double[xcoordinates.size()];
					//double[] pontok_y= new double[ycoordinates.size()];
					System.out.println("POLYGONRAJZOLAS");
					Point2D.Double[] poligonpontok=new Point2D.Double[ycoordinates.size()];
					for(int l=0;l<ycoordinates.size();l++){
						poligonpontok[l]=new Point2D.Double(xcoordinates.get(l),ycoordinates.get(l));
					}
					/*
		 	        for(int j=0;j<ycoordinates.size();j++){
		 	        	pontok_x[j]=xcoordinates.get(j);
		 	        	pontok_y[j]=ycoordinates.get(j);
		 	        	System.out.println(pontok_x[j] + " " +pontok_y[j]);
		 	        }*/
					MyPolygon v=new MyPolygon(poligonpontok,ycoordinates.size());
					components.add(v);
				}
			}

			for(int i=0;i<circles.size();i++){
				if(!circles.get(i).equals("")){
					String[] strings = circles.get(i).split("\\*");
					String nPoint_x = "";
					String nPoint_y = "";

					int k;
					for(k=1;k<strings[0].length() && isSzam(strings[0].charAt(k));){
						nPoint_x+=strings[0].charAt(k);
						k++;
					}
					k++;
					System.out.println(nPoint_x);
					for(;k<strings[0].length() && isSzam(strings[0].charAt(k));){
						nPoint_y+=strings[0].charAt(k);
						k++;
					}
					double x=Double.parseDouble(nPoint_x);
					double y=Double.parseDouble(nPoint_y);
					Point2D.Double newPoint = new Point2D.Double(x,y);
					double r = Double.parseDouble(strings[1]);

					MyCircle c= new MyCircle(newPoint,r);
					components.add(c);
				}
			}

		}
		catch (IOException ioe) {
			System.err.println(ioe.getMessage());
		}

		repaint();
	}

	public Boolean isIsSaved(){
		return isSaved;
	}

	public void setIsSaved(boolean issaved){
		isSaved = issaved;
	}

	public void drawCoordinateSystem(Graphics2D g2, int Cartesian_height){
		g2.drawLine(A.x, A.y, C.x, C.y);
		g2.drawLine(B.x, B.y, D.x, D.y);

		//Draw the arrows
		g2.drawLine(A.x-arrowwidth, A.y+arrowlength, A.x, A.y);	//A1 A
		g2.drawLine(B.x-arrowlength, B.y-arrowwidth, B.x, B.y);	//B1 B
		g2.drawLine(A.x+arrowwidth, A.y+arrowlength, A.x, A.y);	//A2 A
		g2.drawLine(B.x-arrowlength, B.y+arrowwidth, B.x, B.y);	//B2 B
		g2.drawLine(C.x+arrowwidth, C.y-arrowlength, C.x, C.y);	//C1 C
		g2.drawLine(D.x+arrowlength, D.y-arrowwidth, D.x, D.y);	//D1 D
		g2.drawLine(C.x-arrowwidth, C.y-arrowlength, C.x, C.y);	//C2 C
		g2.drawLine(D.x+arrowlength, D.y+arrowwidth, D.x, D.y);	//D2 D

		if(isgrid){
			//Draw lines
			g2.setColor(new Color(210,210,210));
			for(int i=1;(O.x+i*rateSpace)<=B.x;i++){
				g2.drawLine(O.x+i*rateSpace,A.y,O.x+i*rateSpace,C.y);
			}
			for(int i=1;(O.x-i*rateSpace)>=D.x;i++){
				g2.drawLine(O.x-i*rateSpace,A.y,O.x-i*rateSpace,C.y);
			}

			g2.setColor(new Color(210,210,210));
			for(int i=1;(O.y+i*rateSpace)<=C.y;i++){
				g2.drawLine(D.x,O.y+i*rateSpace,B.x,O.y+i*rateSpace);
			}
			for(int i=1;(O.x-i*rateSpace)>=A.y;i++){
				g2.drawLine(D.x,O.y-i*rateSpace,B.x,O.y-i*rateSpace);
			}
		}



		g2.setColor(Color.BLACK);
		//Draw ratelines
		if(rateSpace!=10){

			for(int i=0;(O.y-i*10)>(space+arrowlength);i++){
				g2.drawLine(O.x-rateLine, O.y-i*10, O.x+rateLine, O.y-i*10);
			}
			for(int i=0;(O.y+i*10)<(Cartesian_height-(space + arrowlength));i++){
				g2.drawLine(O.x-rateLine, O.y+i*10, O.x+rateLine, O.y+i*10);
			}
			for(int i=0;(O.x+i*10)<(B.x-arrowlength);i++){
				g2.drawLine(O.x+i*10, O.y-rateLine, O.x+i*10, O.y+rateLine);
			}
			for(int i=0;(O.x-i*10)>(D.x+arrowlength);i++){
				g2.drawLine(O.x-i*10, O.y-rateLine, O.x-i*10, O.y+rateLine);
			}

			//Draw the ratelines

			for(int i=0;(O.y-i*rateSpace)>(space+arrowlength);i++){
				g2.drawLine(O.x-10, O.y-i*rateSpace, O.x+10, O.y-i*rateSpace);
			}
			for(int i=0;(O.y+i*rateSpace)<(Cartesian_height-(space + arrowlength));i++){
				g2.drawLine(O.x-10, O.y+i*rateSpace, O.x+10, O.y+i*rateSpace);
			}
			for(int i=0;(O.x+i*rateSpace)<(B.x-arrowlength);i++){
				g2.drawLine(O.x+i*rateSpace, O.y-10, O.x+i*rateSpace, O.y+10);
			}
			for(int i=0;(O.x-i*rateSpace)>(D.x+arrowlength);i++){
				g2.drawLine(O.x-i*rateSpace, O.y-10, O.x-i*rateSpace, O.y+10);
			}


		}
		else{
			//Draw the ratelines
			g2.setColor(Color.BLACK);
			for(int i=0;(O.y-i*rateSpace)>(space+arrowlength);i++){
				g2.drawLine(O.x-rateLine, O.y-i*rateSpace, O.x+rateLine, O.y-i*rateSpace);
			}
			for(int i=0;(O.y+i*rateSpace)<(Cartesian_height-(space + arrowlength));i++){
				g2.drawLine(O.x-rateLine, O.y+i*rateSpace, O.x+rateLine, O.y+i*rateSpace);
			}
			for(int i=0;(O.x+i*rateSpace)<(B.x-arrowlength);i++){
				g2.drawLine(O.x+i*rateSpace, O.y-rateLine, O.x+i*rateSpace, O.y+rateLine);
			}
			for(int i=0;(O.x-i*rateSpace)>(D.x+arrowlength);i++){
				g2.drawLine(O.x-i*rateSpace, O.y-rateLine, O.x-i*rateSpace, O.y+rateLine);
			}
		}
	}


	@Override
	public void paint(Graphics g){
		super.paint(g);
		Graphics2D mainGraphics = (Graphics2D) g;

		int Cartesian_width = getWidth();
		int Cartesian_height = getHeight();

		A = new Point(Cartesian_width/2,space);
		B = new Point(Cartesian_width-space,Cartesian_height/2);
		C = new Point(Cartesian_width/2,Cartesian_height-space);
		D = new Point(space,Cartesian_height/2);
		O = new Point(Cartesian_width/2, Cartesian_height/2);

		drawCoordinateSystem(mainGraphics,Cartesian_height);

		int pointLine = 2;

		mainGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		if(components!=null){

			for(int i=0;i<components.size();i++){
				if(components.get(i) instanceof Line2D.Double){

					Point A_new=calculatePoint(((Line2D.Double)components.get(i)).getP1(), rateSpace,O);
					Point B_new=calculatePoint(((Line2D.Double)components.get(i)).getP2(), rateSpace,O);
					//g2.drawLine(A_new.x,A_new.y, B_new.x, B_new.y);

					Line2D line2d=new Line2D.Double(A_new.x, A_new.y, B_new.x, B_new.y);

					mainGraphics.draw(line2d);
					//lines.add((Line2D.Double) components.get(i));
				}
				else if(components.get(i) instanceof Point2D.Double){
					Point2D.Double A_new=(Point2D.Double)components.get(i);
					System.out.println(A_new.x + " " + A_new.y);
					drawPoint(mainGraphics, A_new, pointLine, rateSpace, O);
					//points.add((Point) components.get(i));
				}
				else if(components.get(i) instanceof MyCircle){
					//g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
					drawCircle(mainGraphics, (MyCircle)(components.get(i)), rateSpace, O);
				}
				else if(components.get(i) instanceof MyPolygon){
					drawMyPolygon(mainGraphics, (MyPolygon)(components.get(i)),rateSpace,O);
				}
				else if(components.get(i) instanceof MyFunction){
					((MyFunction)(components.get(i))).drawMyFunction(mainGraphics, rateSpace, O);
				}
			}
		}
		if(coloringTransformedShapes!=null/*transformedShapes!=null*/){
			mainGraphics.setColor(linTransformationColor);

			for(Object obj: coloringTransformedShapes.keySet()){
				mainGraphics.setColor(coloringTransformedShapes.get(obj));
				//}
				//for(int i=0;i<transformedShapes.size();i++){

				if(/*transformedShapes.get(i)*/obj instanceof Line2D.Double){

					/*Point A_new=calculatePoint(((Line2D.Double)transformedShapes.get(i)).getP1(), rateSpace,O);
					Point B_new=calculatePoint(((Line2D.Double)transformedShapes.get(i)).getP2(), rateSpace,O);
					g2.drawLine(A_new.x,A_new.y, B_new.x, B_new.y);*/

					Point A_new=calculatePoint(((Line2D.Double)obj).getP1(), rateSpace,O);
					Point B_new=calculatePoint(((Line2D.Double)obj).getP2(), rateSpace,O);
					mainGraphics.drawLine(A_new.x,A_new.y, B_new.x, B_new.y);


					//lines.add((Line2D.Double) components.get(i));
				}
				else if(/*transformedShapes.get(i)*/obj instanceof Point2D.Double){
					/*Point2D.Double A_new=(Point2D.Double)transformedShapes.get(i);
					System.out.println(A_new.x + " " + A_new.y);
					drawPoint(g2, A_new, pointLine, rateSpace, O);*/

					Point2D.Double A_new=(Point2D.Double)obj;
					System.out.println(A_new.x + " " + A_new.y);
					drawPoint(mainGraphics, A_new, pointLine, rateSpace, O);


					//points.add((Point) components.get(i));
				}
				else if(/*transformedShapes.get(i)*/obj instanceof MyCircle){
					//drawCircle(g2, (MyCircle)(transformedShapes.get(i)), rateSpace, O);
					drawCircle(mainGraphics, (MyCircle)(obj), rateSpace, O);
				}
				else if(/*transformedShapes.get(i)*/obj instanceof MyPolygon){
					//drawMyPolygon(g2, (MyPolygon)(transformedShapes.get(i)),rateSpace,O);
					drawMyPolygon(mainGraphics, (MyPolygon)(obj),rateSpace,O);
				}
				else if(/*transformedShapes.get(i)*/obj instanceof MyFunction){

					//((MyFunction)(transformedShapes.get(i))).drawMyFunction(g2, rateSpace, O);
					((MyFunction)obj).drawMyFunction(mainGraphics, rateSpace, O);
				}


			}

			mainGraphics.setColor(Color.BLACK);
		}
		if(/*nonLinearTransformedShapes!=null*/coloringNonLinTransformedShapes!=null){
			mainGraphics.setColor(nonlinTransformationColor);

			for(Object obj: coloringNonLinTransformedShapes.keySet()){
				//for(int i=0;i<nonLinearTransformedShapes.size();i++){
				mainGraphics.setColor(coloringNonLinTransformedShapes.get(obj));
				if(/*nonLinearTransformedShapes.get(i)*/obj instanceof TransformedLine){

					List<Point> lista= new ArrayList<Point>();
					for(int j=0;j<((TransformedLine)obj).numberOfpoints;j++){
						System.out.print(((TransformedLine)obj).line.get(j).toString()+',');
						System.out.println();
						lista.add(calculatePoint(((TransformedLine)obj).line.get(j),rateSpace,O));
					}

					for(int j=1;j<lista.size();j++){
						System.out.print(lista.get(j).toString()+',');
						mainGraphics.drawLine(lista.get(j-1).x, lista.get(j-1).y, lista.get(j).x, lista.get(j).y);
					}
				}
				else if(/*nonLinearTransformedShapes.get(i)*/obj instanceof TransformedShape ){

					List<Point> lista= new ArrayList<Point>();
					for(int j=0;j<((TransformedShape)obj).numberOfpoints;j++){

						lista.add(calculatePoint(((TransformedShape)obj).shape.get(j),rateSpace,O));
					}

					for(int j=1;j<lista.size();j++){

						mainGraphics.drawLine(lista.get(j-1).x, lista.get(j-1).y, lista.get(j).x, lista.get(j).y);
					}
				}
				else if(/*nonLinearTransformedShapes.get(i)*/obj instanceof Point2D.Double){
					//drawPoint(g2, (Point2D.Double)nonLinearTransformedShapes.get(i), pointLine, rateSpace, O);
					drawPoint(mainGraphics, (Point2D.Double)obj, pointLine, rateSpace, O);
				}

			}
			mainGraphics.setColor(Color.BLACK);
		}
	}

	public void drawMyPolygon(Graphics2D g2, MyPolygon polygon, int rateSpace, Point O){
		int n=polygon.n;
		Point[] pontok = new Point[n];

		for(int j=0;j<n;j++){
			pontok[j] = calculatePoint((polygon).pontok[j],rateSpace,O);
		}
		for(int i=1;i<polygon.n;i++){
			g2.drawLine(pontok[i-1].x, pontok[i-1].y, pontok[i].x, pontok[i].y);
		}
		g2.drawLine(pontok[0].x,pontok[0].y,pontok[n-1].x,pontok[n-1].y);
	}

	public void drawCircle(Graphics2D g2, MyCircle circle, int rateSpace, Point O){

		List<Point> pointList= new ArrayList<Point>();
		for(int j=0;j<circle.pointsOfCircle.size();j++){
			pointList.add(calculatePoint(circle.pointsOfCircle.get(j),rateSpace,O));
		}

		for(int i=1;i<pointList.size();i++){
			g2.drawLine(pointList.get(i-1).x, pointList.get(i-1).y, pointList.get(i).x, pointList.get(i).y);
		}
		g2.drawLine(pointList.get(0).x,pointList.get(0).y,pointList.get(pointList.size()-1).x,pointList.get(pointList.size()-1).y);

	}

	public void linearTransformation(int m1, int m2, int m3, int m4){

		if(components!=null){
			isTransformed=true;
			Transformation transzformation = new Transformation();
			List<Object> tmpList=transzformation.linearTransformationWorkspace(components, m1, m2, m3, m4);

			if(coloringTransformedShapes==null){
				coloringTransformedShapes=new HashMap<Object, Color>();
			}

			for(int i=0;i<tmpList.size();i++){
				coloringTransformedShapes.put(tmpList.get(i), linTransformationColor);
				//transformedShapes.add(tmpList.get(i));
			}

			repaint();

		}
		else{
			JOptionPane.showMessageDialog(this,
					"Nincs transzformálható alakzat",
					"Transzformációs hiba",
					JOptionPane.ERROR_MESSAGE);
		}

	}

	public void nonlinearTransformation(String expression1,String expression2){

		if(components!=null){
			isTransformed=true;
			Transformation transformation = new Transformation();
			try {
				List<Object> tmpList=transformation.nonlinearTransformationOfWorkspace(components, expression1,expression2);

				if(coloringNonLinTransformedShapes==null){
					coloringNonLinTransformedShapes=new HashMap<Object, Color>();
				}
				for(int i=0;i<tmpList.size();i++){
					coloringNonLinTransformedShapes.put(tmpList.get(i), nonlinTransformationColor);
				}
				repaint();

			} catch(NumberFormatException e) {
				JOptionPane.showMessageDialog(this,
						"Hibás bemenet",
						"Transzformációs hiba",
						JOptionPane.ERROR_MESSAGE);

			}
		}
		else{
			JOptionPane.showMessageDialog(this,
					"Nincs transzformálható alakzat",
					"Transzformációs hiba",
					JOptionPane.ERROR_MESSAGE);
		}
	}
}
